@@ -6,6 +6,7 @@ from typing import (
     Callable,
     Literal,
     Pattern,
+    Self,
     Sequence,
     TypedDict,
     Union,
@@ -23,7 +24,7 @@ from starlette.responses import Response, StreamingResponse
 
 from lihil.asgi import ASGIBase
 from lihil.auth.oauth import AuthBase
-from lihil.config import AppConfig, SyncDeps
+from lihil.config import AppConfig, lhl_get_config
 from lihil.constant.resp import METHOD_NOT_ALLOWED_RESP
 from lihil.errors import InvalidParamTypeError
 from lihil.interface import (
@@ -152,16 +153,7 @@ class Endpoint[R]:
         else:
             self._plugin_items = ()
 
-        self._static = not any(
-            (
-                self._sig.path_params,
-                self._sig.query_params,
-                self._sig.header_params,
-                self._sig.body_param,
-                self._sig.dependencies,
-                self._sig.plugins,
-            )
-        )
+        self._static = self._sig.static
 
         scoped_by_config = bool(self._props and self._props.scoped is True)
 
@@ -193,7 +185,9 @@ class Endpoint[R]:
                 )
         return params
 
-    async def make_static_call(self, scope: IScope, receive: IReceive, send: ISend):
+    async def make_static_call(
+        self, scope: IScope, receive: IReceive, send: ISend
+    ) -> R | Response:
         try:
             return await self._func()
         except Exception as exc:
@@ -240,24 +234,29 @@ class Endpoint[R]:
 
     def return_to_response(self, raw_return: Any) -> Response:
         if isinstance(raw_return, Response):
-            resp = raw_return
-        elif isgenerator(raw_return) or isasyncgen(raw_return):
-            if isgenerator(raw_return) and not isasyncgen(raw_return):
-                encode_wrapper = syncgen_encode_wrapper(raw_return, self._encoder)
-            else:
-                encode_wrapper = agen_encode_wrapper(raw_return, self._encoder)
-            resp = StreamingResponse(
-                encode_wrapper,
-                media_type="text/event-stream",
-                status_code=self._status_code,
-            )
+            return raw_return
         else:
-            resp = Response(
-                content=self._encoder(raw_return),
-                media_type=self._media_type,
-                status_code=self._status_code,
-            )
-        return resp
+            if isasyncgen(raw_return):
+                encode_wrapper = agen_encode_wrapper(raw_return, self._encoder)
+                resp = StreamingResponse(
+                    encode_wrapper,
+                    media_type="text/event-stream",
+                    status_code=self._status_code,
+                )
+            elif isgenerator(raw_return):
+                encode_wrapper = syncgen_encode_wrapper(raw_return, self._encoder)
+                resp = StreamingResponse(
+                    encode_wrapper,
+                    media_type="text/event-stream",
+                    status_code=self._status_code,
+                )
+            else:
+                resp = Response(
+                    content=self._encoder(raw_return),
+                    media_type=self._media_type,
+                    status_code=self._status_code,
+                )
+            return resp
 
     async def __call__(self, scope: IScope, receive: IReceive, send: ISend) -> None:
         if self._scoped:
@@ -267,25 +266,111 @@ class Endpoint[R]:
             return await response(scope, receive, send)
         if self._static:  # when there is no params at all
             raw_return = await self.make_static_call(scope, receive, send)
+            # response = StaticResponse(raw_return)
         else:
             raw_return = await self.make_call(scope, receive, send, self._graph)
         response = self.return_to_response(raw_return)
         await response(scope, receive, send)
 
 
-class Route(ASGIBase):
-    _flyweights: dict[str, "Route"] = {}
+class RouteBase(ASGIBase):
+    _flyweights: dict[str, "Self"] = {}
 
-    def __new__(cls, path: str = "", **_):
+    def __new__(cls, path: str = "", **_) -> Self:
         p = trim_path(path)
         if p_route := cls._flyweights.get(p):
             return p_route
 
         cls._flyweights[p] = route = super().__new__(cls)
         if parent := cls._flyweights.get(get_parent_path(p)):
-            parent.subroutes.append(route)
+            if route not in parent.subroutes:
+                parent.subroutes.append(route)
         return route
 
+    def __init__(  # type: ignore
+        self,
+        path: str = "",
+        *,
+        graph: Graph | None = None,
+        registry: MessageRegistry | None = None,
+        listeners: list[Callable[..., Any]] | None = None,
+        middlewares: list[MiddlewareFactory[Any]] | None = None,
+    ):
+        super().__init__(middlewares)
+        self.path = trim_path(path)
+        self.path_regex: Pattern[str] | None = None
+        self.graph = graph or Graph(self_inject=False)
+        self.registry = registry or MessageRegistry(event_base=Event, graph=graph)
+        if listeners:
+            self.registry.register(*listeners)
+        self.busterm = BusTerminal(self.registry, graph=graph)
+        self.app_config: AppConfig | None = None
+        self.subroutes: list[Self] = []
+
+    def __truediv__(self, path: str) -> "Self":
+        return self.sub(path)
+
+    async def __call__(self, scope: IScope, receive: IReceive, send: ISend): ...
+
+    def is_direct_child_of(self, other_path: "Route | str") -> bool:
+        if isinstance(other_path, Route):
+            return self.is_direct_child_of(other_path.path)
+
+        if not self.path.startswith(other_path):
+            return False
+        rest = self.path.removeprefix(other_path)
+        return rest.count("/") < 2
+
+    def sub(self, path: str) -> "Self":
+        sub_path = trim_path(path)
+        merged_path = merge_path(self.path, sub_path)
+        for sub in self.subroutes:
+            if sub.path == merged_path:
+                return sub
+        sub = self.__class__(
+            path=merged_path,
+            graph=self.graph,
+            registry=self.registry,
+            middlewares=self.middle_factories,
+        )
+        self.subroutes.append(sub)
+        return sub
+
+    def match(self, scope: IScope) -> IScope | None:
+        path = scope["path"]
+        if not self.path_regex or not (m := self.path_regex.match(path)):
+            return None
+        scope["path_params"] = m.groupdict()
+        return scope
+
+    def add_nodes[T](
+        self, *nodes: Union[IDependent[T], tuple[IDependent[T], INodeConfig]]
+    ) -> None:
+        self.graph.add_nodes(*nodes)
+
+    def factory[R](self, node: Callable[..., R], **node_config: Unpack[INodeConfig]):
+        return self.graph.node(node, **node_config)
+
+    def listen(self, listener: Callable[[Event, Any], None]) -> None:
+        self.registry.register(listener)
+
+    def has_listener(self, listener: Callable[..., Any]) -> bool:
+        event_metas = list(self.registry.event_mapping.values())
+
+        for metas in event_metas:
+            for meta in metas:
+                if meta.handler is listener:
+                    return True
+        return False
+
+    def setup(self, graph: Graph | None = None, busterm: BusTerminal | None = None):
+        self.graph = graph or self.graph
+        self.busterm = busterm or self.busterm
+        self.app_config = lhl_get_config()
+
+
+class Route(RouteBase):
+
     def __init__(  # type: ignore
         self,
         path: str = "",
@@ -296,18 +381,15 @@ class Route(ASGIBase):
         middlewares: list[MiddlewareFactory[Any]] | None = None,
         props: EndpointProps | None = None,
     ):
-        super().__init__(middlewares)
-        self.path = trim_path(path)
-        self.path_regex: Pattern[str] | None = None
+        super().__init__(
+            path,
+            graph=graph,
+            registry=registry,
+            listeners=listeners,
+            middlewares=middlewares,
+        )
         self.endpoints: dict[HTTP_METHODS, Endpoint[Any]] = {}
-        self.graph = graph or Graph(self_inject=False)
-        self.registry = registry or MessageRegistry(event_base=Event, graph=graph)
-        if listeners:
-            self.registry.register(*listeners)
-        self.busterm = BusTerminal(self.registry, graph=graph)
-        self.subroutes: list[Route] = []
         self.call_stacks: dict[HTTP_METHODS, ASGIApp] = {}
-        self.app_config: AppConfig | None = None
         self.props = props or EndpointProps(tags=[generate_route_tag(self.path)])
 
     def __repr__(self):
@@ -317,27 +399,12 @@ class Route(ASGIBase):
         )
         return f"{self.__class__.__name__}({self.path!r}{endpoints_repr})"
 
-    def __truediv__(self, path: str) -> "Route":
-        return self.sub(path)
-
     async def __call__(self, scope: IScope, receive: IReceive, send: ISend) -> None:
         endpoint = self.call_stacks.get(scope["method"]) or METHOD_NOT_ALLOWED_RESP
         await endpoint(scope, receive, send)
 
-    def is_direct_child_of(self, other_path: "Route | str") -> bool:
-        if isinstance(other_path, Route):
-            return self.is_direct_child_of(other_path.path)
-
-        if not self.path.startswith(other_path):
-            return False
-        rest = self.path.removeprefix(other_path)
-        return rest.count("/") < 2
-
-    def setup(self, **deps: Unpack[SyncDeps]):
-        if deps:
-            self.app_config = deps.get("app_config") or self.app_config
-            self.graph = deps.get("graph") or self.graph
-            self.busterm = deps.get("busterm") or self.busterm
+    def setup(self, graph: Graph | None = None, busterm: BusTerminal | None = None):
+        super().setup(graph, busterm)
 
         for method, ep in self.endpoints.items():
             ep.setup()
@@ -356,25 +423,6 @@ class Route(ASGIBase):
         else:
             raise KeyError(f"{method_func} is not in current route")
 
-    def sub(self, path: str) -> "Route":
-        sub_path = trim_path(path)
-        current_path = merge_path(self.path, sub_path)
-        sub = Route(path=current_path, graph=self.graph)
-        self.subroutes.append(sub)
-        return sub
-
-    def match(self, scope: IScope) -> IScope | None:
-        path = scope["path"]
-        if not self.path_regex or not (m := self.path_regex.match(path)):
-            return None
-        scope["path_params"] = m.groupdict()
-        return scope
-
-    def add_nodes[T](
-        self, *nodes: Union[IDependent[T], tuple[IDependent[T], INodeConfig]]
-    ) -> None:
-        self.graph.add_nodes(*nodes)
-
     def redirect(self, method: MethodType, **path_params: str) -> None:
         # owner = method.__self__
         raise NotImplementedError
@@ -404,21 +452,6 @@ class Route(ASGIBase):
                 self.path_regex = build_path_regex(self.path)
         return func
 
-    def factory[R](self, node: Callable[..., R], **node_config: Unpack[INodeConfig]):
-        return self.graph.node(node, **node_config)
-
-    def listen(self, listener: Callable[[Event, Any], None]) -> None:
-        self.registry.register(listener)
-
-    def has_listener(self, listener: Callable[..., Any]) -> bool:
-        event_metas = list(self.registry.event_mapping.values())
-
-        for metas in event_metas:
-            for meta in metas:
-                if meta.handler is listener:
-                    return True
-        return False
-
     # ============ Http Methods ================
 
     @overload
