@@ -21,7 +21,7 @@ from ididi import DependentNode, Graph, INode, NodeConfig, Resolver
 from ididi.config import USE_FACTORY_MARK
 from ididi.utils.param_utils import MISSING as IDIDI_MISSING
 from ididi.utils.typing_utils import is_builtin_type
-from msgspec import DecodeError
+from msgspec import UNSET, DecodeError
 from msgspec import Meta as ParamConstraint
 from msgspec import Struct, ValidationError, convert, field
 from msgspec.structs import fields as get_fields
@@ -36,9 +36,11 @@ from lihil.interface import (
     Maybe,
     ParamBase,
     ParamLocation,
+    RegularTypes,
     is_provided,
 )
 from lihil.interface.marks import (
+    HEADER_REQUEST_MARK,
     JW_TOKEN_RETURN_MARK,
     ParamMarkType,
     Struct,
@@ -56,20 +58,22 @@ from lihil.problems import (
     ValidationProblem,
 )
 from lihil.utils.json import decoder_factory
-from lihil.utils.string import parse_header_key
+from lihil.utils.string import parse_header_key, to_kebab_case
 from lihil.utils.typing import (
     get_origin_pro,
     is_mapping_type,
     is_nontextual_sequence,
     is_union_type,
 )
-from lihil.vendor_types import FormData, Headers, QueryParams, Request, UploadFile
+from lihil.vendors import FormData, Headers, QueryParams, Request, UploadFile, WebSocket
 
-type RequestParam[T] = PathParam[T] | QueryParam[T] | HeaderParam[T]
+type RequestParam[T] = PathParam[T] | QueryParam[T] | HeaderParam[T] | CookieParam[T]
 type ParsedParam[T] = RequestParam[T] | BodyParam[T] | DependentNode | PluginParam
 
 
-LIHIL_DEPENDENCIES: tuple[type, ...] = (Request, EventBus, Resolver)
+STARLETTE_TYPES: tuple[type, ...] = (Request, WebSocket)
+LIHIL_TYPES: tuple[type, ...] = (EventBus, Resolver)
+LIHIL_PRIMITIVES: tuple[type, ...] = STARLETTE_TYPES + LIHIL_TYPES
 
 
 def is_file_body(annt: Any) -> TypeGuard[type[UploadFile]]:
@@ -259,6 +263,11 @@ class HeaderParam[T](QueryParam[T]):
     location: ClassVar[ParamLocation] = "header"
 
 
+class CookieParam[T](HeaderParam[T], kw_only=True):
+    alias = "cookie"
+    cookie_name: str
+
+
 class BodyParam[T](Decodable[bytes | FormData, T], kw_only=True):
     location: ClassVar[ParamLocation] = "body"
     content_type: BodyContentType = "application/json"
@@ -279,8 +288,23 @@ class BodyParam[T](Decodable[bytes | FormData, T], kw_only=True):
         return self.validate(body)
 
 
+"""
 class ParamMetas(Record):
+    mark_type: Maybe[ParamMarkType] = MISSING
+    metas: tuple[Any, ...] = ()
 
+class RequestParamMeta(ParamMetas)
+    custom_decoder: Maybe[IDecoder[Any, Any]] = MISSING
+    constraint: ParamConstraint | None = None
+
+class NodeParamMeta(ParamMetas):
+    factory: Maybe[INode[..., Any]]
+    node_config: NodeConfig
+"""
+
+
+class ParamMetas(Record):
+    # TODO: specification, HeaderParamMeta, NodeParamMeta, etc.
     metas: tuple[Any, ...] = ()
     custom_decoder: IDecoder[Any, Any] | None = None
     mark_type: ParamMarkType | None = None
@@ -290,7 +314,7 @@ class ParamMetas(Record):
 
     @classmethod
     def from_metas(cls, metas: list[Any]) -> "ParamMetas":
-        current_mark_type = None
+        current_mark_type = None  # TODO: default to query
         custom_decoder = None
         factory = None
         config = None
@@ -300,7 +324,7 @@ class ParamMetas(Record):
             if isinstance(meta, CustomDecoder):
                 custom_decoder = meta
             elif mark_type := extract_mark_type(meta):
-                if current_mark_type and mark_type is not current_mark_type:
+                if current_mark_type and (mark_type != current_mark_type):
                     raise NotSupportedError("can't use more than one param mark")
                 current_mark_type = mark_type
             elif meta == USE_FACTORY_MARK:  # TODO: use PluginParser
@@ -388,6 +412,19 @@ def req_param_factory[T](
             default=default,
         )
     elif location == "header":
+        if alias == "cookie":
+            assert param_metas
+            cookie_name = param_metas.metas[0]
+            cookie_name = parse_header_key(name, cookie_name)
+            return CookieParam(
+                name=name,
+                cookie_name=cookie_name,
+                alias=alias,
+                type_=param_type,
+                annotation=annotation,
+                decoder=decoder,
+                default=default,
+            )
         return HeaderParam(
             name=name,
             alias=alias,
@@ -430,21 +467,21 @@ class ParamParser:
             self.seen = set()
 
         # mark_type or param_type, dict[str | type, PluginProvider]
-        self.plugin_types = LIHIL_DEPENDENCIES
+        self.lhl_primitives = LIHIL_PRIMITIVES
         self.app_config = app_config
 
-    def is_plugin_type(self, param_type: Any) -> TypeGuard[type]:
+    def is_lhl_primitive(self, param_type: Any) -> TypeGuard[type]:
         "Dependencies that should be injected and managed by lihil"
         if not isinstance(param_type, type):
             param_origin = get_origin(param_type)
             if param_origin is Union:
-                return any(self.is_plugin_type(arg) for arg in get_args(param_type))
+                return any(self.is_lhl_primitive(arg) for arg in get_args(param_type))
             elif param_origin and is_builtin_type(param_origin):
                 return False
             else:
-                return self.is_plugin_type(type(param_type))
+                return self.is_lhl_primitive(type(param_type))
         else:
-            return issubclass(param_type, self.plugin_types)
+            return issubclass(param_type, self.lhl_primitives)
 
     def _parse_rule_based[T](
         self,
@@ -469,12 +506,20 @@ class ParamParser:
                 param_metas=param_metas,
                 location="path",
             )
-        elif self.is_plugin_type(param_type):
-            return [
-                PluginParam(
-                    type_=param_type, annotation=annotation, name=name, default=default
+        elif self.is_lhl_primitive(param_type):
+            plugin = PluginParam(
+                type_=param_type, annotation=annotation, name=name, default=default
+            )
+            plugins = [plugin]
+
+            if param_metas and param_metas.factory:  # Annotated[Dep, use(dep_factory)]
+                assert param_metas.node_config
+                node = self.graph.analyze(
+                    param_metas.factory, config=param_metas.node_config
                 )
-            ]
+                plugins += self._parse_node(node)
+
+            return cast(list[ParsedParam[Any]], plugins)
         elif is_body_param(param_type):
             if is_file_body(param_type):
                 req_param = file_body_param(
@@ -559,7 +604,7 @@ class ParamParser:
         else:
             custom_decoder = param_metas.custom_decoder
             if custom_decoder is None:
-                if self.app_config is None or self.app_config.security is None:
+                if self.app_config is None or self.app_config.security is UNSET:
                     raise MissingDependencyError("security config")
                 sec_config = self.app_config.security
                 secret = sec_config.jwt_secret
@@ -584,6 +629,79 @@ class ParamParser:
 
         return req_param
 
+    def _parse_header[T](
+        self,
+        name: str,
+        type_: type[T] | UnionType,
+        annotation: Any,
+        default: Maybe[T],
+        param_metas: ParamMetas,
+    ) -> ParsedParam[T]:
+        location = "header"
+        pmetas = param_metas.metas
+        if not pmetas:
+            header_key = to_kebab_case(name)
+        else:
+            mark_idx = pmetas.index(HEADER_REQUEST_MARK)
+            key_meta = pmetas[mark_idx - 1]
+            header_key = parse_header_key(name, key_meta).lower()
+
+        if header_key == "authorization":
+            return self._parse_auth_header(
+                name=name,
+                header_key=header_key,
+                type_=type_,
+                annotation=annotation,
+                default=default,
+                param_metas=param_metas,
+            )
+        else:
+            param_alias = header_key
+
+        return req_param_factory(
+            name=name,
+            alias=param_alias,
+            param_type=type_,
+            annotation=annotation,
+            location=location,
+            default=default,
+            param_metas=param_metas,
+        )
+
+    def _parse_body[T](
+        self,
+        name: str,
+        param_alias: str,
+        type_: type[T] | UnionType,
+        annotation: Any,
+        default: Maybe[T],
+        mark_type: Literal["form", "body"],
+        custom_decoder: IDecoder[Any, Any] | None,
+    ) -> BodyParam[T]:
+        if mark_type == "form":
+            content_type = "multipart/form-data"
+            decoder = custom_decoder or formdecoder_factory(type_)
+            body_param = BodyParam(
+                name=name,
+                alias=param_alias,
+                type_=type_,
+                annotation=annotation,
+                default=default,
+                decoder=decoder,
+                content_type=content_type,
+            )
+        else:
+            decoder = custom_decoder or decoder_factory(type_)
+            body_param = BodyParam(
+                name=name,
+                alias=param_alias,
+                type_=type_,
+                annotation=annotation,
+                default=default,
+                decoder=decoder,
+            )
+        return body_param
+
     def _parse_marked[T](
         self,
         name: str,
@@ -604,47 +722,25 @@ class ParamParser:
             # Easy case, Pure non-deps request params with param marks.
             location: ParamLocation
             param_alias = name
-            content_type: BodyContentType = "application/json"
 
             if mark_type == "header":
-                location = "header"
-                header_key = parse_header_key(name, param_metas.metas).lower()
-                if header_key == "authorization":
-                    return self._parse_auth_header(
-                        name=name,
-                        header_key=header_key,
-                        type_=type_,
-                        annotation=annotation,
-                        default=default,
-                        param_metas=param_metas,
-                    )
-                else:
-                    param_alias = header_key
-            elif mark_type == "body":
-                decoder = custom_decoder or decoder_factory(type_)
-                body_param = BodyParam(
+                return self._parse_header(
                     name=name,
-                    alias=param_alias,
                     type_=type_,
                     annotation=annotation,
                     default=default,
-                    decoder=decoder,
-                    content_type=content_type,
+                    param_metas=param_metas,
                 )
-                return body_param
-            elif mark_type == "form":
-                content_type = "multipart/form-data"
-                decoder = custom_decoder or formdecoder_factory(type_)
-                body_param = BodyParam(
-                    name=name,
-                    alias=param_alias,
+            elif mark_type in ("body", "form"):
+                return self._parse_body(
+                    name,
+                    param_alias=param_alias,
                     type_=type_,
                     annotation=annotation,
                     default=default,
-                    decoder=decoder,
-                    content_type=content_type,
+                    mark_type=mark_type,
+                    custom_decoder=custom_decoder,
                 )
-                return body_param
 
             elif mark_type == "path":
                 location = "path"
@@ -665,7 +761,7 @@ class ParamParser:
     def _parse_plugin_from_meta(
         self,
         name: str,
-        type_: type | UnionType,
+        type_: RegularTypes,
         annotation: type[Any] | UnionType | GenericAlias | TypeAliasType,
         default: Maybe[Any],
         metas: list[Any] | None,
@@ -699,6 +795,7 @@ class ParamParser:
         default: Maybe[T] = LIHIL_MISSING,
     ) -> list[ParsedParam[T]]:
         parsed_type, pmetas = get_origin_pro(annotation)
+        parsed_type = cast(type[T], parsed_type)
 
         if plugins := self._parse_plugin_from_meta(
             name, parsed_type, annotation, default, pmetas
