@@ -8,10 +8,11 @@ from lihil.config import AppConfig
 from lihil.interface import Base, IEncoder, Record
 from lihil.problems import ValidationProblem
 from lihil.utils.string import find_path_keys
-from lihil.vendor_types import FormData, Headers, QueryParams, Request
+from lihil.vendors import FormData, Headers, QueryParams, Request, cookie_parser, WebSocket
 
 from .params import (
     BodyParam,
+    CookieParam,
     HeaderParam,
     ParamParser,
     PathParam,
@@ -70,8 +71,29 @@ class EndpointSignature[R](Base):
         verrors: list[Any] = []
         params: dict[str, Any] = {}
 
+        if req_header:
+            raw_cookies: str | None = None
+            cookie_params: dict[str, str] | None = None
+            for name, param in self.header_params.items():
+                if param.alias == "cookie":
+                    cookie: CookieParam[Any] = param  # type: ignore
+
+                    if raw_cookies is None:
+                        raw_cookie = req_header["cookie"]
+                    if cookie_params is None:
+                        cookie_params = cookie_parser(raw_cookie)
+
+                    raw_cookie: str = cookie_params[cookie.cookie_name]
+                    val, error = param.validate(raw_cookie)
+                else:
+                    val, error = param.extract(req_header)
+
+                if val:
+                    params[name] = val
+                else:
+                    verrors.append(error)
+
         zipped = (
-            (req_header, self.header_params),
             (req_path, self.path_params),
             (req_query, self.query_params),
         )
@@ -98,7 +120,7 @@ class EndpointSignature[R](Base):
         parsed_result = ParseResult(params, verrors)
         return parsed_result
 
-    def parse_query(self, req: Request) -> ParseResult:
+    def parse_query(self, req: Request | WebSocket) -> ParseResult:
         req_path = req.path_params if self.path_params else None
         req_query = req.query_params if self.query_params else None
         req_header = req.headers if self.header_params else None
@@ -119,6 +141,19 @@ class EndpointSignature[R](Base):
             params = self.prepare_params(req_path, req_query, req_header, body)
         return params
 
+    @property
+    def static(self) -> bool:
+        return not any(
+            (
+                self.path_params,
+                self.query_params,
+                self.header_params,
+                self.body_param,
+                self.dependencies,
+                self.plugins,
+            )
+        )
+
     @classmethod
     def from_function[FR](
         cls,
