@@ -1,5 +1,5 @@
 import uuid
-from types import UnionType
+from types import GenericAlias, UnionType
 from typing import Annotated, Any, Literal
 
 import pytest
@@ -7,10 +7,12 @@ from ididi import AsyncScope, Graph, Ignore, use
 from starlette.requests import Request
 
 from lihil import (
+    Cookie,
     Empty,
     Form,
     Header,
     Json,
+    Meta,
     Payload,
     Query,
     Request,
@@ -97,7 +99,8 @@ def test_status_conflict(rusers: Route):
 
     rusers.get(get_user)
     with pytest.raises(StatusConflictError):
-        rusers.get_endpoint(get_user).setup()
+        ep = rusers.get_endpoint(get_user)
+        ep.setup()
 
 
 def test_annotated_generic(rusers: Route):
@@ -204,9 +207,10 @@ async def test_scoped_endpoint(rusers: Route, lc: LocalClient):
     await lc.call_endpoint(ep)
 
 
+@pytest.mark.debug
 async def test_ep_drop_body(rusers: Route, lc: LocalClient):
 
-    async def get() -> Resp[Empty, 204]:
+    async def get() -> Resp[Empty, 400]:
         return "asdf"
 
     rusers.get(get)
@@ -214,6 +218,7 @@ async def test_ep_drop_body(rusers: Route, lc: LocalClient):
 
     res = await lc.call_endpoint(ep)
 
+    assert res.status_code == 400
     assert await res.body() == b""
 
 
@@ -611,6 +616,8 @@ async def test_endpoint_with_jwt_fail_without_security_config(
 
 
 async def test_endpoint_login_and_validate(testroute: Route, lc: LocalClient):
+    from lihil.config import lhl_set_config
+
     async def get_me(token: JWTAuth[UserProfile]) -> Resp[Text, status.OK]:
         assert token.user_id == "1" and token.user_name == "2"
         return "ok"
@@ -620,12 +627,12 @@ async def test_endpoint_login_and_validate(testroute: Route, lc: LocalClient):
 
     testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
     testroute.post(login_get_token)
-
-    testroute.setup(
+    lhl_set_config(
         app_config=AppConfig(
             security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
         )
     )
+    testroute.setup()
 
     login_ep = testroute.get_endpoint(login_get_token)
 
@@ -661,11 +668,12 @@ async def test_endpoint_login_and_validate_with_str_resp(
 
     testroute.get(auth_scheme=OAuth2PasswordFlow(token_url="token"))(get_me)
     testroute.post(login_get_token)
-    testroute.setup(
-        app_config=AppConfig(
+    set_config(
+        AppConfig(
             security=SecurityConfig(jwt_secret="mysecret", jwt_algorithms=["HS256"])
         )
     )
+    testroute.setup()
 
     login_ep = testroute.get_endpoint(login_get_token)
 
@@ -747,7 +755,11 @@ async def test_plugin_without_processor(testroute: Route):
 
     class MyPlugin(PluginBase):
         def parse(
-            self, name: str, type_: type | UnionType, annotation: Any, default: Any
+            self,
+            name: str,
+            type_: type | UnionType | GenericAlias,
+            annotation: Any,
+            default: Any,
         ) -> PluginParam:
             return PluginParam(
                 type_=type_,
@@ -808,3 +820,63 @@ async def test_endpoint_with_tuple_query():
 def test_set_1d_iterable():
     for t in (set, frozenset, tuple, list):
         assert is_nontextual_sequence(t)
+
+
+async def test_ep_with_constraints():
+    called: bool = False
+
+    async def get_user(
+        n: Annotated[int, Meta(gt=0)], user_id: Annotated[str, Meta(min_length=5)]
+    ):
+        nonlocal called
+        called = True
+
+    lc = LocalClient()
+
+    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    resp = await lc(ep, path_params={"user_id": "user"}, query_params={"n": -1})
+    res = await resp.json()
+    assert not called
+
+
+async def test_ep_with_cookie():
+    called: bool = False
+
+    async def get_user(
+        refresh_token: Annotated[
+            Cookie[str, Literal["refresh-token"]], Meta(min_length=1)
+        ],
+        user_id: Annotated[str, Meta(min_length=5)],
+    ):
+        nonlocal called
+        assert len(user_id) >= 5
+        called = True
+        return True
+
+    lc = LocalClient(headers={"cookie": "refresh-token=asdf"})
+
+    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    resp = await lc(ep, path_params={"user_id": "user123"})
+    res = await resp.json()
+    assert res
+    assert called
+
+
+async def test_ep_with_cookie2():
+    called: bool = False
+
+    async def get_user(
+        refresh_token: Annotated[Cookie[str], Meta(min_length=1)],
+        user_id: Annotated[str, Meta(min_length=5)],
+    ):
+        nonlocal called
+        called = True
+        return True
+
+    lc = LocalClient(headers={"cookie": "refresh-token=asdf"})
+
+    ep = lc.make_endpoint(get_user, path="/{user_id}")
+    resp = await lc(ep, path_params={"user_id": "user123"})
+    res = await resp.json()
+    assert res
+    assert called
