@@ -230,6 +230,7 @@ def test_analyze_markedparam_query(param_parser: ParamParser):
 
 
 # Test analyze_markedparam for Header
+@pytest.mark.debug
 def test_analyze_markedparam_header(param_parser: ParamParser):
     result = param_parser.parse_param("user_agent", Header[str])
     assert len(result) == 1
@@ -432,7 +433,7 @@ def test_param_provider_with_invalid_mark(param_parser):
 
 
 def test_param_provider_with_invalid_plugin(param_parser: ParamParser):
-    assert not param_parser.is_plugin_type(5)
+    assert not param_parser.is_lhl_primitive(5)
 
 
 def test_path_param_with_default_fail(param_parser: ParamParser):
@@ -554,8 +555,35 @@ def test_constraint_dt(param_parser: ParamParser):
     assert isinstance(dt, datetime) and dt.tzinfo
 
 
-
 def test_param_with_bytes_in_union(param_parser: ParamParser):
 
     with pytest.raises(NotSupportedError):
         res = param_parser.parse_param("n", int | bytes)
+
+
+from typing import Literal
+
+from starlette.requests import Request
+
+from lihil.interface.marks import HEADER_REQUEST_MARK, Cookie
+from lihil.utils.typing import get_origin_pro
+
+
+def test_parse_cookie(param_parser: ParamParser):
+
+    t, meta = get_origin_pro(Header[str, "ads_id"])
+    assert t == str and meta == ["ads_id", HEADER_REQUEST_MARK]
+
+    t, meta = get_origin_pro(Cookie[str, Literal["ads_id"]])
+
+    res = param_parser.parse_param("cookies", Cookie[str, "ads_id"])[0]
+    assert res.cookie_name == "ads_id"
+
+    def cookie_decoder(x):
+        x
+
+    res = param_parser.parse_param(
+        "cookies", Annotated[Cookie[str, "ads_id"], CustomDecoder(cookie_decoder)]
+    )[0]
+    assert res.cookie_name == "ads_id"
+    assert res.decoder is cookie_decoder
